# node-natnet

A client for the Optitrack NatNet client in Node.js. The implementation uses the official NatNetSDK. Using this module it is possible the gather frame data from the *Motive* software by enabling *Data Streaming* in it.

<blockquote>
  This module only supports Windows operating systems!
</blockquote>

## Usage

``` js
const { NatNetClient } = require('node-natnet')

const client = new NatNetClient()

client.initialize({
  local: '127.0.0.1',
  server: '127.0.0.1'
})

client.on('frame', data => {
  console.log('Received frame data: ', data)
})
```

## Install

Install from NPM:

```
$ npm install node-natnet
```

Install from the Git repository:
```
$ git clone https://github.com/gerhardberger/node-natnet.git
$ cd node-natnet
$ npm install
```
Then in your code import it with its path: `require('path/to/folder')`.

### Compiling manually

The module comes with prebuilt binaries for various Node.js versions, but if your's not supported, you can compile it manually, by running `npm run compile` inside the `node-natnet` folder after `npm install`.

## API Reference

### Methods

#### `const client = new NatNetClient(type)`

Creates a new `NatNetClient` instance. `type` is an optional *String* parameter that determines the connection (this can be configured inside *Motive*).

The possible connection types are:
- `'unicast'`, default
- `'multicast'`.

#### `client.initialize(options)`

Initializes and connects to a NatNet server that can be started in *Motive*.

The options object can contain:
- `local`: *String*, the local IP address that the client is running on. **Required.**
- `server`: *String*, the server IP address that the data is streamed to. **Required.**
- `hostCommandPort`: *Number*, the custom command port if there is one. *Optional.*
- `hostDataPort`: *Number*, the custom data port if there is one. *Optional.*

#### `client.uninitialize()`

Disconnects the client application from the server.

### Events

#### `client.on('frame', callback)`

This event is emitted every time there is new frame data received from *Motive*. The rate of this is dependent on the FPS counter in *Motive*.

The callback has one parameter which is an object containing the current frame data:
- `id`: The current frame identifier.
- `markerSets`: A named collection of identified markers and the marker positions `{ x, y, z }`.
- `otherMarkers`: List of point cloud solved 3D positions `{ x, y, z }` for all markers in the frame that are not labeled.
- `labeledMarkers`: Ordered, padded, point cloud solved, model filled (where occluded) labeled marker data.
- `latency`: The latency value of the frame.
- `timestamp`: Timing information for the frame.

### Properties

#### `client.connected`

A *Boolean* value that indicates wether the client is connected to the server.

#### `client.serverDescription`

Fetches the server description through a NatNet command. Returns an *Object*:
- `hostPresent`: *Boolean*
- `hostComputerName`: *String*
- `hostComputerAddress`: *String*
- `hostApp`: *String*
- `appVersion`: *String*
- `natNetVersion`: *String*

#### `client.dataDescriptions`

Fetches the data descriptions through a NatNet command. Returns an *[Object]*:
- `name`: *String*
- `markerNames`: *String*
