#include "TypeConverter.h"

using namespace v8;

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sServerDescription* serverDescription) {
  std::string hostComputerName =
    std::string(serverDescription->szHostComputerName);

  char* addr = (char*)malloc(1024);
  sprintf(addr, "%d.%d.%d.%d",
    serverDescription->HostComputerAddress[0],
    serverDescription->HostComputerAddress[1],
    serverDescription->HostComputerAddress[2],
    serverDescription->HostComputerAddress[3]);
  std::string hostComputerAddress = std::string(addr);
  free(addr);


  std::string hostApp =
    std::string(serverDescription->szHostApp);

  char* appV = (char*)malloc(1024);
  sprintf(appV, "%d.%d.%d.%d",
    serverDescription->HostAppVersion[0],
    serverDescription->HostAppVersion[1],
    serverDescription->HostAppVersion[2],
    serverDescription->HostAppVersion[3]);
  std::string appVersion = std::string(appV);
  free(appV);

  char* natNetV = (char*)malloc(1024);
  sprintf(natNetV, "%d.%d.%d.%d",
    serverDescription->NatNetVersion[0],
    serverDescription->NatNetVersion[1],
    serverDescription->NatNetVersion[2],
    serverDescription->NatNetVersion[3]);
  std::string natNetVersion = std::string(natNetV);
  free(natNetV);

  huron::Dictionary dict = huron::Dictionary::CreateEmpty(isolate);
	dict.Set("hostPresent", serverDescription->HostPresent);
  dict.Set("hostComputerName", hostComputerName);
  dict.Set("hostComputerAddress", hostComputerAddress);
  dict.Set("hostApp", hostApp);
  dict.Set("appVersion", appVersion);
  dict.Set("natNetVersion", natNetVersion);

  return huron::ConvertToV8(isolate, dict);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sMarkerSetDescription* markerSetDescription) {
  std::string name = std::string(markerSetDescription->szName);

  Local<Array> markerNames = Nan::New<Array>();
  for (int j = 0; j < markerSetDescription->nMarkers; ++j) {
    Nan::Set(markerNames, j, huron::ConvertToV8(isolate,
      std::string(markerSetDescription->szMarkerNames[j])));
  }

  huron::Dictionary dict = huron::Dictionary::CreateEmpty(isolate);
  dict.Set("type", "marker-set");
  dict.Set("name", name);
  dict.Set("markerNames", markerNames);

  return huron::ConvertToV8(isolate, dict);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sFrameOfMocapData* frameData) {
  huron::Dictionary dict = huron::Dictionary::CreateEmpty(isolate);
  return natnet::Converter::ToV8(isolate, frameData, dict);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sFrameOfMocapData* frameData,
    huron::Dictionary& result) {
  if (frameData == NULL) {
    return huron::ConvertToV8(isolate, result);
  }

  Local<Array> markerSets = Nan::New<Array>();
  for (int i = 0; i < frameData->nMarkerSets; ++i) {
    Nan::Set(markerSets, i,
      natnet::Converter::ToV8(isolate, &frameData->MocapData[i]));
  }

  result.Set("id", frameData->iFrame);
  result.Set("markerSets", markerSets);
  result.Set("otherMarkers", natnet::Converter::ToV8(isolate,
    frameData->nOtherMarkers, frameData->OtherMarkers));
  result.Set("rigidBodies", natnet::Converter::ToV8(isolate,
    frameData->nRigidBodies, frameData->RigidBodies));
  result.Set("skeletons", natnet::Converter::ToV8(isolate,
    frameData->nSkeletons, frameData->Skeletons));
  result.Set("labeledMarkers", natnet::Converter::ToV8(isolate,
    frameData->nLabeledMarkers, frameData->LabeledMarkers));
  result.Set("latency", frameData->fLatency);
  result.Set("timecode", frameData->Timecode);
  result.Set("timecodeSubframe", frameData->TimecodeSubframe);
  result.Set("timestamp", frameData->fTimestamp);
  result.Set("params", (int)frameData->params);

  return huron::ConvertToV8(isolate, result);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sMarkerSetData* markerSetData) {

  std::string name = std::string(markerSetData->szName);

  huron::Dictionary result = huron::Dictionary::CreateEmpty(isolate);
  result.Set("name", name);
  result.Set("markers", natnet::Converter::ToV8(isolate,
    markerSetData->nMarkers, markerSetData->Markers));

  return huron::ConvertToV8(isolate, result);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const MarkerData markerData) {
  huron::Dictionary result = huron::Dictionary::CreateEmpty(isolate);
  result.Set("x", markerData[0]);
  result.Set("y", markerData[1]);
  result.Set("z", markerData[2]);

  return huron::ConvertToV8(isolate, result);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const size_t n,
    const MarkerData* markerData) {
  Local<Array> markerDatas = Nan::New<Array>();

  for (size_t i = 0; i < n; ++i) {
    Nan::Set(markerDatas, i,
      natnet::Converter::ToV8(isolate, markerData[i]));
  }

  return markerDatas;
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sMarker* marker) {
  huron::Dictionary result = huron::Dictionary::CreateEmpty(isolate);
  result.Set("id", marker->ID);
  result.Set("x", marker->x);
  result.Set("y", marker->y);
  result.Set("z", marker->z);
  result.Set("size", marker->size);
  result.Set("params", (int)marker->params);

  return huron::ConvertToV8(isolate, result);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const size_t n,
    const sMarker* markers) {
  Local<Array> markerDatas = Nan::New<Array>();

  for (size_t i = 0; i < n; ++i) {
    Nan::Set(markerDatas, i, natnet::Converter::ToV8(isolate, &markers[i]));
  }

  return markerDatas;
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sRigidBodyData* rigidBodyData) {
  Local<Array> markerIDs = Nan::New<Array>();
  Local<Array> markerSizes = Nan::New<Array>();

  for (int i = 0; i < rigidBodyData->nMarkers; ++i) {
    Nan::Set(markerIDs, i, huron::ConvertToV8(isolate,
      rigidBodyData->MarkerIDs[i]));
    Nan::Set(markerSizes, i, huron::ConvertToV8(isolate,
      rigidBodyData->MarkerSizes[i]));
  }

  huron::Dictionary dict = huron::Dictionary::CreateEmpty(isolate);
  dict.Set("id", rigidBodyData->ID);
  dict.Set("x", rigidBodyData->x);
  dict.Set("y", rigidBodyData->y);
  dict.Set("z", rigidBodyData->z);
  dict.Set("qx", rigidBodyData->qx);
  dict.Set("qy", rigidBodyData->qy);
  dict.Set("qz", rigidBodyData->qz);
  dict.Set("qw", rigidBodyData->qw);
  dict.Set("markers", natnet::Converter::ToV8(isolate, rigidBodyData->nMarkers,
    rigidBodyData->Markers));
  dict.Set("markerIDs", markerIDs);
  dict.Set("markerSizes", markerSizes);
  dict.Set("meanError", rigidBodyData->MeanError);
  dict.Set("params", (int)rigidBodyData->params);
  
  return huron::ConvertToV8(isolate, dict);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const size_t n,
    const sRigidBodyData* rigidBodyData) {
  Local<Array> rigidBodyDatas = Nan::New<Array>();

  for (size_t i = 0; i < n; ++i) {
    Nan::Set(rigidBodyDatas, i, natnet::Converter::ToV8(isolate,
      &rigidBodyData[i]));
  }

  return rigidBodyDatas;
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const sSkeletonData* skeletonData) {
  huron::Dictionary dict = huron::Dictionary::CreateEmpty(isolate);
  dict.Set("id", skeletonData->skeletonID);
  dict.Set("rigidBodies", natnet::Converter::ToV8(isolate,
    skeletonData->nRigidBodies, skeletonData->RigidBodyData));
  
  return huron::ConvertToV8(isolate, dict);
}

Local<Value> natnet::Converter::ToV8(
    Isolate* isolate,
    const size_t n,
    const sSkeletonData* skeletonData) {
  Local<Array> skeletonDatas = Nan::New<Array>();

  for (size_t i = 0; i < n; ++i) {
    Nan::Set(skeletonDatas, i, natnet::Converter::ToV8(isolate,
      &skeletonData[i]));
  }

  return skeletonDatas;
}
