const test = require('tape')
const { NatNetClient } = require('../')

test('natnet client constructor with default connection type', t => {
  const client = new NatNetClient()

  t.equal(client.type, 'unicast')
  t.end()
})

test('natnet client constructor with unicast connection type', t => {
  const client = new NatNetClient('unicast')

  t.equal(client.type, 'unicast')
  t.end()
})

test('natnet client constructor with multicast connection type', t => {
  const client = new NatNetClient('multicast')

  t.equal(client.type, 'multicast')
  t.end()
})

test('natnet client inited with only addresses successful connection', t => {
  const client = new NatNetClient()

  client.initialize({ local: '127.0.0.1', server: '127.0.0.1' })

  t.ok(client.connected)
  t.end()
})

test('natnet client inited with command port successful connection', t => {
  const client = new NatNetClient()

  client.initialize({
    local: '127.0.0.1',
    server: '127.0.0.1',
    hostCommandPort: 1234
  })

  t.ok(client.connected)
  t.end()
})

test('natnet client inited with host and command port successful connection', t => {
  const client = new NatNetClient()

  client.initialize({
    local: '127.0.0.1',
    server: '127.0.0.1',
    hostCommandPort: 1234,
    hostDataPort: 2345
  })

  t.ok(client.connected)
  t.end()
})

test('natnet client inited and successful uninitialization', t => {
  const client = new NatNetClient()
  client.initialize({ local: '127.0.0.1', server: '127.0.0.1' })

  client.uninitialize()

  t.false(client.connected)
  t.end()
})

test('natnet client server description check', t => {
  const client = new NatNetClient()
  client.initialize({ local: '127.0.0.1', server: '127.0.0.1' })

  const description = client.serverDescription

  t.ok(description.hostPresent)
  t.end()
})

test('natnet client data descriptions check', t => {
  const client = new NatNetClient()
  client.initialize({ local: '127.0.0.1', server: '127.0.0.1' })

  const descriptions = client.dataDescriptions

  t.ok(descriptions instanceof Array)
  t.end()
})

test('natnet client frame data emitted', t => {
  const client = new NatNetClient()

  client.on('frame', data => {
    console.log(data)
    t.pass()
    t.end()
  })

  client.initialize({ local: '127.0.0.1', server: '127.0.0.1' })
})
